function derK = FitNIGScale_DerK(K, y, nu, a, b)

% assume a NIG model with parameters (nu, K, a, b) for observations y
% find the derivative of the log-lik wrt K, evaluated at K
% so that we can find the zeros of this derivative as a max likelihood
% estimate of K
[R, N] = size(y);

derK  = 0;
ybar = mean(y);
tmp = zeros(1,N);
for i = 1 : N
    [logP, PostMu, PostK, PostA, PostB] = GetLogPYNormalIGamma1Dim(y(:,i), nu, K, a, b);

    mustar = ybar(i);
    ssqstar = max(var(y(:,i)), 1e-06);
    derBeta = 0.5*(ybar(i) - nu)^2*R*R/((K + R)^2);
    derMu = R*(nu - ybar(i)) / ((K + R)^2);

    % evaluate the derivative analytically
    tmp(i) = 0.5/K - (a + 0.5*R)*derBeta*PostB - 0.5/(K + R) + ...
        (derBeta - 0.5*(mustar - nu)^2 + 0.5*(mustar - PostMu)^2 + (K + R)*(PostMu - mustar)*derMu) / ssqstar;
    derK = derK + tmp(i);
end


