#include <math.h>
#include <string.h>
#include <limits.h>
#include "SafeArray.h"
#include "defs.h"
#include "utils.h"
#include "NormalIGamma.h"
#include "NIGPeriodic.h"
#include "mex.h"

void GetSPLogP (numtype *LogPY,
				numtype *LogPKGivenY,
				numtype *LogPYGivenK, 
				numtype *ThetaPR,
				numtype *ThetaDE,
				numtype *ThetaBG,
				const numtype *DataMatr,
				const numtype *NRepl,
				const numtype *CycleDep,
				const numtype *NCycleDep,
				const numtype *PComp,
				const numtype *PRPriorMu,
				const numtype *PRPriorK0,
				const numtype *PRPriorA1,
				const numtype *PRPriorA2,
				const numtype *DEPriorMu,
				const numtype *DEPriorK0,
				const numtype *DEPriorA1,
				const numtype *DEPriorA2,
				const numtype *BGPriorMu,
				const numtype *BGPriorK0,
				const numtype *BGPriorA1,
				const numtype *BGPriorA2,
				const int N,
				const int DNMaxRepl,
				const int D,
				const int T, 
				const int NMaxCycleDep)
{

	SafeArray<const numtype> DataMatr_(DataMatr, N, DNMaxRepl);
	SafeArray<const numtype> NRepl_(NRepl, N, D);

	SafeArray<const numtype> CycleDep_(CycleDep, T, NMaxCycleDep);
	SafeArray<const numtype> NCycleDep_(NCycleDep, T);
	
	SafeArray<numtype> ThetaBG_(ThetaBG, N, 2);
	SafeArray<numtype> ThetaDE_(ThetaDE, N, D, 2);
	SafeArray<numtype> ThetaPR_(ThetaPR, N, T, 2);
	
	SafeArray<numtype> LogPYGivenK_(LogPYGivenK, N,3);

	int NMaxRepl = DNMaxRepl / D;

	numtype *da = new numtype[DNMaxRepl];
	numtype *daJ = new numtype[DNMaxRepl];

	memset(LogPYGivenK, 0, N*3*sizeof(numtype));

	int i, j, r, k, nDa, nDaJ, t;
	numtype tmp, tmpMu, tmpSsq, x, PostMu, PostK0, PostA1, PostA2;
	//% likelihood under the BG model
    for (i = 1; i <= N; i++)
	{
		nDa = 0;
		for (j = 1; j <= D; j++)
		{
			nDaJ = 0;
			for (r = 1; r <= NRepl_(i,j); r++)
			{
				x = DataMatr_(i,r + (j-1)*NMaxRepl);
				if (x != -Inf)
				{
					da[nDa++] = x;
					daJ[nDaJ++] = x;
				}
			}
			// Get marginal likelihood / sample for DE model 
			GetLogPYNormalIGamma1Dim(&tmp, &PostMu, &PostK0, &PostA1, &PostA2, 
				daJ, DEPriorMu[j-1], DEPriorK0[j-1], DEPriorA1[j-1], DEPriorA2[j-1], nDaJ);
			LogPYGivenK_(i,2) += tmp;
			
			//% sample
			NIGSample(&tmpMu, &tmpSsq, PostMu, PostK0, PostA1, PostA2, 1, 1);
	
			ThetaDE_(i,j,1) = tmpMu;
			ThetaDE_(i,j,2) = tmpSsq;
		}	
	    GetLogPYNormalIGamma1Dim(&tmp, &PostMu, &PostK0, &PostA1, &PostA2, 
			da, *BGPriorMu, *BGPriorK0, *BGPriorA1, *BGPriorA2, nDa);
		LogPYGivenK_(i,3) = tmp;
		
		//% sample
		NIGSample(&tmpMu, &tmpSsq, PostMu, PostK0, PostA1, PostA2, 1, 1);

        ThetaBG_(i,1) = tmpMu;
		ThetaBG_(i,2) = tmpSsq;

		// periodic compoenent
		for (j = 1; j <= T; j++)
		{
			nDaJ = 0;
			for (k = 1; k <= NCycleDep[j-1]; k++)
			{
				t = CycleDep_(j,k);
				for (r = 1; r <= NRepl_(i,j); r++)
				{
					x = DataMatr_(i,r + (t-1)*NMaxRepl);
					if (x != -Inf)
					{
						daJ[nDaJ++] = x;
					}
				}
			}
			// Get marginal likelihood / sample for DE model 
			GetLogPYNormalIGamma1Dim(&tmp, &PostMu, &PostK0, &PostA1, &PostA2, 
				daJ, PRPriorMu[j-1], PRPriorK0[j-1], PRPriorA1[j-1], PRPriorA2[j-1], nDaJ);
			LogPYGivenK_(i,1) += tmp;
				
			//% sample
			NIGSample(&tmpMu, &tmpSsq, PostMu, PostK0, PostA1, PostA2, 1, 1);
		
			ThetaPR_(i,j,1) = tmpMu;
			ThetaPR_(i,j,2) = tmpSsq;
		}	
	}

	delete[] da;
	delete[] daJ;

	InvertProb2 (LogPY, LogPKGivenY, LogPYGivenK, PComp, 3, N) ;
}

void GetSPMultiCompLogP (numtype *LogPY,
				numtype *LogPKGivenY,
				numtype *LogPYGivenK, 
				numtype *Mu,
				numtype *RV,
				const numtype *DataMatr,
				const numtype *NRepl,
				const numtype *CycleDep,
				const numtype *NCycleDep,
				const numtype *PComp,
				const numtype *CompType,
				const numtype *PriorMu,
				const numtype *PriorK0,
				const numtype *PriorA1,
				const numtype *PriorA2,
				const int N,
				const int DNMaxRepl,
				const int D,
				const int T,
				const int K,
				const int NMaxCycleDep)
{

	SafeArray<const numtype> DataMatr_(DataMatr, N, DNMaxRepl);
	SafeArray<const numtype> NRepl_(NRepl, N, D);

	SafeArray<const numtype> PriorMu_(PriorMu, K, D);
	SafeArray<const numtype> PriorK0_(PriorK0, K, D);
	SafeArray<const numtype> PriorA1_(PriorA1, K, D);
	SafeArray<const numtype> PriorA2_(PriorA2, K, D);

	SafeArray<const numtype> CycleDep_(CycleDep, T, NMaxCycleDep);
	SafeArray<const numtype> NCycleDep_(NCycleDep, T);

	SafeArray<const numtype> CompType_(CompType, K);
	
	SafeArray<numtype> Mu_(Mu, N, D, K);
	SafeArray<numtype> RV_(RV, N, D, K);
	
	SafeArray<numtype> LogPYGivenK_(LogPYGivenK, N,K);

	int NMaxRepl = DNMaxRepl / D;

	numtype *da = new numtype[DNMaxRepl];
	numtype *daJ = new numtype[DNMaxRepl];

	memset(LogPYGivenK, 0, N*K*sizeof(numtype));

	int i, j, r, k, nDa, nDaJ, t, c;
	numtype tmp, tmpMu, tmpSsq, x, PostMu, PostK0, PostA1, PostA2;
	//% likelihood under the BG model
    for (i = 1; i <= N; i++)
	{
		nDa = 0;
		for (j = 1; j <= D; j++)
		{
			nDaJ = 0;
			for (r = 1; r <= NRepl_(i,j); r++)
			{
				x = DataMatr_(i,r + (j-1)*NMaxRepl);
				if (x != -Inf)
				{
					da[nDa++] = x;
					daJ[nDaJ++] = x;
				}
			}
			// Get marginal likelihood / sample for DE model 
			for (k = 1; k <= K; k++)
			{
				if (CompType_(k) == 2)
				{
					GetLogPYNormalIGamma1Dim(&tmp, &PostMu, &PostK0, &PostA1, &PostA2, 
						daJ, PriorMu_(k,j), PriorK0_(k,j), PriorA1_(k,j), PriorA2_(k,j), nDaJ);
					LogPYGivenK_(i,k) += tmp;
					
					//% sample
					NIGSample(&tmpMu, &tmpSsq, PostMu, PostK0, PostA1, PostA2, 1, 1);
			
					Mu_(i,j,k) = tmpMu;
					RV_(i,j,k) = tmpSsq;
				}
			}
		}	
		for (k = 1; k <= K; k++)
		{
			if (CompType_(k) == 3)
			{
			    GetLogPYNormalIGamma1Dim(&tmp, &PostMu, &PostK0, &PostA1, &PostA2, 
					da, PriorMu_(k,1), PriorK0_(k,1), PriorA1_(k,1), PriorA2_(k,1), nDa);
				LogPYGivenK_(i,k) = tmp;
		
				//% sample
				NIGSample(&tmpMu, &tmpSsq, PostMu, PostK0, PostA1, PostA2, 1, 1);
		
		        Mu_(i,1,k) = tmpMu;
				RV_(i,1,k) = tmpSsq;
			}
		}

		// periodic compoenent
		for (j = 1; j <= T; j++)
		{
			nDaJ = 0;
			for (c = 1; c <= NCycleDep[j-1]; c++)
			{
				t = CycleDep_(j,c);
				for (r = 1; r <= NRepl_(i,j); r++)
				{
					x = DataMatr_(i,r + (t-1)*NMaxRepl);
					if (x != -Inf)
					{
						daJ[nDaJ++] = x;
					}
				}
			}
			for (k = 1; k <= K; k++)
			{
				if (CompType_(k) == 1)
				{
					// Get marginal likelihood / sample for DE model 
					GetLogPYNormalIGamma1Dim(&tmp, &PostMu, &PostK0, &PostA1, &PostA2, 
						daJ, PriorMu_(k,j), PriorK0_(k,j), PriorA1_(k,j), PriorA2_(k,j), nDaJ);
					LogPYGivenK_(i,k) += tmp;
						
					//% sample
					NIGSample(&tmpMu, &tmpSsq, PostMu, PostK0, PostA1, PostA2, 1, 1);
				
					Mu_(i,j,k) = tmpMu;
					RV_(i,j,k) = tmpSsq;
				}
			}
		}	
	}

	delete[] da;
	delete[] daJ;

	InvertProb2 (LogPY, LogPKGivenY, LogPYGivenK, PComp, K, N) ;
}
