#include <math.h>
#include <string.h>
#include "SafeArray.h"
#include "defs.h"
#include "utils.h"
#include "mex.h"
#include <limits.h>

void NormalIGammaUpdatePosterior1Dim(numtype *PostMu, numtype *PostSc, numtype *PostA1, numtype *PostA2, const numtype *Data,numtype PriorMu, numtype PriorSc, numtype PriorA1, numtype PriorA2, const int N)
{
	int i;
	if (N == 0)
	{
        *PostMu = PriorMu;
        *PostSc = PriorSc;
        *PostA1 = PriorA1;
        *PostA2 = PriorA2;
	}
    else
	{
        numtype Nu0 = 2*PriorA1;
        numtype S0 = 1/(PriorA1*PriorA2);
    
		numtype M = 0,s = 0;
		for (i = 0; i < N; i++)
		{
			M += Data[i];
		}
		M /= N;
		for (i = 0; i < N; i++)
		{
			s += (Data[i] - M)*(Data[i] - M);
		}
        
        numtype Mu = (PriorSc/(PriorSc + N))*PriorMu + (N/(PriorSc+N))*M;
        numtype Kn = PriorSc+N;
        
        numtype Nun  = Nu0 + N;
        numtype Ssqn = (Nu0*S0 + s + ((PriorSc*N)/(PriorSc+N))*(M-PriorMu)*(M-PriorMu))/Nun;
        
        *PostMu = Mu;
        *PostSc = Kn;
        *PostA1= Nun/2.0;
        *PostA2 = 1/((*PostA1)*Ssqn);
    }
}



void NormalIGammaUpdatePosterior(numtype *PostMu, numtype *PostSc, numtype *PostA1, numtype *PostA2, const numtype *Data, const numtype *PriorMu, const numtype *PriorSc, const numtype *PriorA1, const numtype *PriorA2, const int N, const int D, const int GZ)
{
	if (N == 0)
	{
		memcpy(PostMu, PriorMu, sizeof(numtype)*GZ*D);
		memcpy(PostSc, PriorSc, sizeof(numtype)*GZ*D);
		memcpy(PostA1, PriorA1, sizeof(numtype)*GZ*D);
		memcpy(PostA2, PriorA2, sizeof(numtype)*GZ*D);

		return;
	}

	SafeArray<numtype> PostMu_(PostMu, GZ, D);
	SafeArray<numtype> PostSc_(PostSc, GZ, D);
	SafeArray<numtype> PostA1_(PostA1, GZ, D);
	SafeArray<numtype> PostA2_(PostA2, GZ, D);

	SafeArray<const numtype> PriorMu_(PriorMu, GZ, D);
	SafeArray<const numtype> PriorSc_(PriorSc, GZ, D);
	SafeArray<const numtype> PriorA1_(PriorA1, GZ, D);
	SafeArray<const numtype> PriorA2_(PriorA2, GZ, D);

	SafeArray<const numtype> Data_(Data, N, D);
	int i, j, z;
	for (z = 1; z <= GZ; z++)
	{
		for (j = 1; j <= D; j++)
		{
		        numtype Nu0 = 2*PriorA1_(z,j);
				numtype S0 = 1/(PriorA1_(z,j)*PriorA2_(z,j));
		    
				numtype M = 0,s = 0;
				for (i = 1; i <= N; i++)
				{
					M += Data_(i,j);
				}
				M /= N;
				for (i = 1; i <= N; i++)
				{
					s += (Data_(i,j) - M)*(Data_(i,j) - M);
				}	
				
				numtype Mu = (PriorSc_(z,j)/(PriorSc_(z,j) + N))*PriorMu_(z,j) + (N/(PriorSc_(z,j)+N))*M;
				numtype Kn = PriorSc_(z,j)+N;
	        
				numtype Nun  = Nu0 + N;
				numtype Ssqn = (Nu0*S0 + s + ((PriorSc_(z,j)*N)/(PriorSc_(z,j)+N))*(M-PriorMu_(z,j))*(M-PriorMu_(z,j)))/Nun;
		        
				PostMu_(z,j) = Mu;
				PostSc_(z,j) = Kn;
				PostA1_(z,j)= Nun/2.0;
				PostA2_(z,j) = 1/(PostA1_(z,j)*Ssqn);
		}
	}
}


void NormalIGammaUpdatePosteriorGZ1(numtype *PostMu, numtype *PostSc, numtype *PostA1, numtype *PostA2, const numtype *Data, const numtype *PriorMu, const numtype *PriorSc, const numtype *PriorA1, const numtype *PriorA2, const int N, const int D)
{
	SafeArray<const numtype> Data_(Data, N, D);

	int i, j;
	if (N == 0)
	{
		memcpy(PostMu, PriorMu, sizeof(numtype)*D);
		memcpy(PostSc, PriorSc, sizeof(numtype)*D);
		memcpy(PostA1, PriorA1, sizeof(numtype)*D);
		memcpy(PostA2, PriorA2, sizeof(numtype)*D);
	}
	else
	{
		for (j = 0; j < D; j++)
		{
			numtype Nu0 = 2*PriorA1[j];
			numtype S0 = 1/(PriorA1[j]*PriorA2[j]);
		    
			numtype M = 0,s = 0;
			for (i = 1; i <= N; i++)
			{
				M += Data_(i,j+1);
			}
			M /= N;
			for (i = 1; i <= N; i++)
			{
				s += (Data_(i,j+1) - M)*(Data_(i,j+1) - M);
			}	
			
			numtype Mu = (PriorSc[j]/(PriorSc[j] + N))*PriorMu[j] + (N/(PriorSc[j]+N))*M;
			numtype Kn = PriorSc[j]+N;
        
			numtype Nun  = Nu0 + N;
			numtype Ssqn = (Nu0*S0 + s + ((PriorSc[j]*N)/(PriorSc[j]+N))*(M-PriorMu[j])*(M-PriorMu[j]))/Nun;
	        
			PostMu[j] = Mu;
			PostSc[j] = Kn;
			PostA1[j]= Nun/2.0;
			PostA2[j] = 1/(PostA1[j]*Ssqn);
		}
	}
}


void IGammaUpdatePosterior1Dim(
			numtype *PostA1, 
			numtype *PostA2, 
	  const numtype *ObservedSsq, 
	  const numtype PriorA1, 
	  const numtype PriorA2,
	  int N)
{
	SafeArray<const numtype> ObservedSsq_(ObservedSsq,  N);
	numtype sumSsq = 0;
	for (int i = 1; i <= N; i++)
		sumSsq += ObservedSsq_(i);

	*PostA1 = PriorA1 + 0.5*N;
	*PostA2 = 1/(1/PriorA2 + 0.5*sumSsq);
}

void IGammaUpdatePosterior(
			numtype *PostA1, 
			numtype *PostA2, 
	  const numtype *ObservedSsq, 
	  const numtype *PriorA1, 
	  const numtype *PriorA2,
	  int N,
	  int D)
{
	SafeArray<const numtype> PriorA1_(PriorA1, D);
	SafeArray<const numtype> PriorA2_(PriorA2, D);

	SafeArray<const numtype> ObservedSsq_(ObservedSsq, N);

	SafeArray<numtype> PostA1_(PostA1, D);
	SafeArray<numtype> PostA2_(PostA2, D);

	for (int j = 1; j <= D; j++)
	{
		numtype sumSsq = 0;
		for (int i = 1; i <= N; i++)
		{
			sumSsq += ObservedSsq_(i);
		}
	
		PostA1_(j) = PriorA1_(j) + 0.5*N;
		PostA2_(j) = 1/(1/PriorA2_(j) + 0.5*sumSsq);
	}
}

void IGammaGammaUpdatePosterior1Dim(
			numtype *PostA1, 
			numtype *PostA2, 
	  const numtype *Observed, 
	  const numtype IGamA1, 
	  const numtype PriorA1, 
	  const numtype PriorA2,
	  int N)
{
	//% prior model for IGamma variable is Gamma distribution on the second parameter of IGamma and fixed first parameter.
	//% update posterior distribution of the Gamma distribution parameter beta

	SafeArray<const numtype> Observed_(Observed,  N);
	numtype sumInv = 0;
	for (int i = 1; i <= N; i++)
		sumInv += 1/Observed_(i);

	*PostA1 = PriorA1 + N*IGamA1;
	*PostA2 = 1/(1/PriorA2 + sumInv);
}

void IGammaGammaUpdatePosterior(
			numtype *PostA1, 
			numtype *PostA2, 
	  const numtype *Observed, 
	  const numtype *IGamA1, 
	  const numtype *PriorA1, 
	  const numtype *PriorA2,
	  int N,
	  int D)
{
	//% prior model for IGamma variable is Gamma distribution on the second parameter of IGamma and fixed first parameter.
	//% update posterior distribution of the Gamma distribution parameter beta

	SafeArray<const numtype> IGamA1_(IGamA1,  D);
	SafeArray<const numtype> PriorA1_(PriorA1,  D);
	SafeArray<const numtype> PriorA2_(PriorA2, D);

	SafeArray<const numtype> Observed_(Observed, N);

	SafeArray<numtype> PostA1_(PostA1,  D);
	SafeArray<numtype> PostA2_(PostA2,  D);

	for (int j = 1; j <= D; j++)
	{
		numtype sumInv = 0;
		for (int i = 1; i <= N; i++)
		{
			sumInv += 1/Observed_(i);
		}
	
		PostA1_(j) = PriorA1_(j) + N*IGamA1_(j);
		PostA2_(j) = 1/(1/PriorA2_(j) + sumInv);
	}
}

void NormalIGammaUpdatePosteriorS(numtype *PostMu, numtype *PostSc, numtype *PostA1, numtype *PostA2, const numtype *Data, const numtype *DataIdx, const numtype *PriorMu, const numtype *PriorSc, const numtype *PriorA1, const numtype *PriorA2, const int N, const int D, const int NIdx)
{
	SafeArray<numtype> PostMu_(PostMu,  D);
	SafeArray<numtype> PostSc_(PostSc,  D);
	SafeArray<numtype> PostA1_(PostA1,  D);
	SafeArray<numtype> PostA2_(PostA2,  D);

	SafeArray<const numtype> PriorMu_(PriorMu,  D);
	SafeArray<const numtype> PriorSc_(PriorSc,  D);
	SafeArray<const numtype> PriorA1_(PriorA1,  D);
	SafeArray<const numtype> PriorA2_(PriorA2,  D);

	SafeArray<const numtype> Data_(Data, N, D);

	int i, j;
	for (j = 1; j <= D; j++)
	{
		if (NIdx == 0)
		{
	        PostMu_(j) = PriorMu_(j);
			PostSc_(j) = PriorSc_(j);
			PostA1_(j) = PriorA1_(j);
			PostA2_(j) = PriorA2_(j);
		}
		else
		{
			SafeArray<const numtype> DataIdx_(DataIdx, NIdx);
	        numtype Nu0 = 2*PriorA1_(j);
			numtype S0 = 1/(PriorA1_(j)*PriorA2_(j));
	    
			numtype M = 0,s = 0;
			for (i = 1; i <= NIdx; i++)
			{
				M += Data_(int(DataIdx_(i)),j);
			}
			M /= NIdx;
			for (i = 1; i <= NIdx; i++)
			{
				s += (Data_(int(DataIdx_(i)),j) - M)*(Data_(int(DataIdx_(i)),j) - M);
			}
	        
			numtype Mu = (PriorSc_(j)/(PriorSc_(j) + NIdx))*PriorMu_(j) + (NIdx/(PriorSc_(j)+NIdx))*M;
			numtype Kn = PriorSc_(j)+NIdx;
	        
			numtype Nun  = Nu0 + NIdx;
			numtype Ssqn = (Nu0*S0 + s + ((PriorSc_(j)*NIdx)/(PriorSc_(j)+NIdx))*(M-PriorMu_(j))*(M-PriorMu_(j)))/Nun;
	        
			PostMu_(j) = Mu;
			PostSc_(j) = Kn;
			PostA1_(j)= Nun/2.0;
			PostA2_(j) = 1/(PostA1_(j)*Ssqn);
		}
	}
}


void GetLogPYNormalIGamma1Dim(numtype *LogPY, numtype *PostMu, numtype *PostSc, numtype *PostA1, numtype *PostA2, const numtype *Data, numtype PriorMu, numtype PriorSc, numtype PriorA1, numtype PriorA2, const int N)
{
	numtype MuStar, SsqStar, tmp, InvSsqStar, LogInvSsqStar;

	if (N == 0)
	{
		*LogPY = 0;
		*PostMu = PriorMu;
		*PostSc = PriorSc;
		*PostA1 = PriorA1;
		*PostA2 = PriorA2;
	}
	else
	{
		// get posterior
		NormalIGammaUpdatePosterior1Dim(PostMu, PostSc, PostA1, PostA2, Data, PriorMu, PriorSc, PriorA1, PriorA2, N);
	
		//% choose mean values as ThetaStar
		MuStar = PriorMu;
		if (PriorA1 > 1)
    		SsqStar = 1/((PriorA1-1)*PriorA2);
        else
    		SsqStar = 0.1;

		InvSsqStar = 1.0/SsqStar;
		LogInvSsqStar = log(InvSsqStar);

    		
	//	sprintf (err, "GetLogPYNormalIGamma1Dim: MuStar = %g, SsqStar = %g", MuStar, SsqStar);
	//	mexWarnMsgTxt (err);
	
		numtype pMuSsq = 0, pMuSsqGivenY = 0, pYGivenMuSsq = 0;
	
		//% calculate prior
		GetLogPIGamma(&tmp, &SsqStar, PriorA1, PriorA2, 1, 1);
		pMuSsq += tmp;
	
	//	sprintf (err, "pMuSsq = %g", pMuSsq);
	//	mexWarnMsgTxt (err);
	
		pMuSsq += GetLogP1Dim1(MuStar, PriorMu, PriorSc*InvSsqStar, log(PriorSc)+LogInvSsqStar);
	
	//	sprintf (err, "pMuSsq = %g", pMuSsq);
	//	mexWarnMsgTxt (err);
	
		//% calculate posterior
		GetLogPIGamma(&tmp, &SsqStar, (*PostA1), (*PostA2), 1, 1);
		pMuSsqGivenY += tmp;
	
	//	sprintf (err, "pMuSsqGivenY = %g", pMuSsqGivenY);
	//	mexWarnMsgTxt (err);
	
		pMuSsqGivenY += GetLogP1Dim1(MuStar, *PostMu, (*PostSc)*InvSsqStar, log(*PostSc)+LogInvSsqStar);
	
	//	sprintf (err, "pMuSsqGivenY = %g", pMuSsqGivenY);
	//	mexWarnMsgTxt (err);
	
		//% calculate likelihood
		numtype *LogPYGivenTheta = new numtype[N];
		SafeArray<numtype> LogPYGivenTheta_(LogPYGivenTheta, N);
		GetLogP1Dim(LogPYGivenTheta, Data, MuStar, SsqStar, InvSsqStar, 1,N);

		int i;
		for (i = 1; i <= N; i++)
		{
			pYGivenMuSsq += LogPYGivenTheta_(i);
		}
		delete[] LogPYGivenTheta;
		*LogPY = pMuSsq + pYGivenMuSsq  - pMuSsqGivenY;
	}
}


void GetLogPYNormalIGamma(
						  numtype *LogPY, 
						  numtype *LogPYGivenZ,
						  numtype *PostMu, 
						  numtype *PostSc, 
						  numtype *PostA1, 
						  numtype *PostA2, 
						  numtype *PostZ, 
				    const numtype *Data, 
					const numtype *PriorMu, 
					const numtype *PriorSc, 
					const numtype *PriorA1, 
					const numtype *PriorA2, 
					const numtype *PriorZ, 
					const int N, 
					const int D, 
					const int GZ)
{
	SafeArray<numtype> PostMu_(PostMu, GZ, D);
	SafeArray<numtype> PostSc_(PostSc, GZ, D);
	SafeArray<numtype> PostA1_(PostA1, GZ, D);
	SafeArray<numtype> PostA2_(PostA2, GZ, D);

	SafeArray<const numtype> PriorMu_(PriorMu, GZ, D);
	SafeArray<const numtype> PriorSc_(PriorSc, GZ, D);
	SafeArray<const numtype> PriorA1_(PriorA1, GZ, D);
	SafeArray<const numtype> PriorA2_(PriorA2, GZ, D);

	SafeArray<const numtype> Data_(Data, N, D);

	SafeArray<numtype> LogPYGivenZ_(LogPYGivenZ, GZ);

	int i, j, z;
	numtype tmp;
	numtype MuStar, SsqStar, InvSsqStar, LogInvSsqStar, pMuSsq = 0, pMuSsqGivenY = 0, pYGivenMuSsq = 0;

	// get posterior
	NormalIGammaUpdatePosterior(PostMu, PostSc, PostA1, PostA2, Data, PriorMu, PriorSc, PriorA1, PriorA2, N, D, GZ);

	*LogPY = 0;
	for (z = 1; z <= GZ; z++)
	{
		pMuSsq = 0, pMuSsqGivenY = 0, pYGivenMuSsq = 0;
		for (j = 1; j <= D; j++)
		{
			//% choose mean values as ThetaStar
			MuStar = PriorMu_(z,j);
			if (PriorA1_(z,j) > 1)
    			SsqStar = 1/((PriorA1_(z,j)-1)*PriorA2_(z,j));
    		else
    			SsqStar = 0.1;
	
			InvSsqStar = 1.0/SsqStar;
			LogInvSsqStar = log(InvSsqStar);

			//% calculate prior
			GetLogPIGamma(&tmp, &SsqStar, PriorA1_(z,j), PriorA2_(z,j), 1, 1);
			pMuSsq += tmp;
	
			//% calculate posterior
			GetLogPIGamma(&tmp, &SsqStar, PostA1_(z,j), PostA2_(z,j), 1, 1);
			pMuSsqGivenY += tmp;
	
			//% calculate prior
			pMuSsq += GetLogP1Dim1(MuStar, PriorMu_(z,j), PriorSc_(z,j)*InvSsqStar, log(PriorSc_(z,j))+LogInvSsqStar);
	
			//% calculate posterior
			pMuSsqGivenY += GetLogP1Dim1(MuStar, PostMu_(z,j), PostSc_(z,j)*InvSsqStar, log(PostSc_(z,j))+LogInvSsqStar);
	
			//% calculate likelihood
			for (i = 1; i <= N; i++)
			{
				pYGivenMuSsq += GetLogP1Dim1(Data_(i,j), MuStar, InvSsqStar, LogInvSsqStar);
			}
		}

	//	sprintf (err, "pMuSsq = %g", pMuSsq);
	//	mexWarnMsgTxt (err);
	
	
	//	sprintf (err, "pMuSsqGivenY = %g", pMuSsqGivenY);
	//	mexWarnMsgTxt (err);

		LogPYGivenZ_(z) = pMuSsq + pYGivenMuSsq  - pMuSsqGivenY;
	}
	InvertProb(LogPY, PostZ, LogPYGivenZ, PriorZ, GZ);
}

void GetLogPYNormalIGammaGZ1(
						  numtype *LogPY, 
						  numtype *PostMu, 
						  numtype *PostSc, 
						  numtype *PostA1, 
						  numtype *PostA2, 
				    const numtype *Data, 
					const numtype *PriorMu, 
					const numtype *PriorSc, 
					const numtype *PriorA1, 
					const numtype *PriorA2, 
					const int N, 
					const int D)
{
	SafeArray<const numtype> Data_(Data, N, D);

	int i, j;
	numtype tmp;
	numtype MuStar, SsqStar, InvSsqStar, LogInvSsqStar, pMuSsq = 0, pMuSsqGivenY = 0, pYGivenMuSsq = 0;

	// get posterior
	NormalIGammaUpdatePosteriorGZ1(PostMu, PostSc, PostA1, PostA2, Data, PriorMu, PriorSc, PriorA1, PriorA2, N, D);

	*LogPY = 0;

	pMuSsq = 0, pMuSsqGivenY = 0, pYGivenMuSsq = 0;
	for (j = 0; j < D; j++)
	{
		//% choose mean values as ThetaStar
		MuStar = PriorMu[j];
		if (PriorA1[j] > 1)
    		SsqStar = 1/((PriorA1[j]-1)*PriorA2[j]);
    	else
    		SsqStar = 0.1;

		InvSsqStar = 1.0/SsqStar;
		LogInvSsqStar = log(InvSsqStar);

		//% calculate prior

		GetLogPIGamma(&tmp, &SsqStar, PriorA1[j], PriorA2[j], 1, 1);
		pMuSsq += tmp;

		//% calculate posterior
		GetLogPIGamma(&tmp, &SsqStar, PostA1[j], PostA2[j], 1, 1);
		pMuSsqGivenY += tmp;
		
		//% calculate prior
		pMuSsq += GetLogP1Dim1(MuStar, PriorMu[j], PriorSc[j]*InvSsqStar, log(PriorSc[j]) + LogInvSsqStar);
		
		//% calculate posterior
		pMuSsqGivenY += GetLogP1Dim1(MuStar, PostMu[j], PostSc[j]/SsqStar, log(PriorSc[j]) + LogInvSsqStar);

		//% calculate likelihood
		for (i = 1; i <= N; i++)
		{
			pYGivenMuSsq += GetLogP1Dim1(Data_(i,j+1), MuStar, InvSsqStar, LogInvSsqStar);
		}
	}
	//	sprintf (err, "pMuSsq = %g", pMuSsq);
	//	mexWarnMsgTxt (err);

	
	//	sprintf (err, "pMuSsqGivenY = %g", pMuSsqGivenY);
	//	mexWarnMsgTxt (err);

	*LogPY = pMuSsq + pYGivenMuSsq  - pMuSsqGivenY;
}

void GetLogPYNormalIGammaReplS(
						  numtype *LogPY, 
						  numtype *LogPKGivenY, 
				    const numtype *Data, 
					const numtype *NRepl,
					const numtype *PriorMu, 
					const numtype *PriorSc, 
					const numtype *PriorA1, 
					const numtype *PriorA2, 
					const numtype *PK,
					const int K, 
					const int N, 
					const int D,
					const int DNMaxRepl)
{
	int NMaxRepl = DNMaxRepl / D;

	SafeArray<const numtype> PriorMu_(PriorMu, K, D);
	SafeArray<const numtype> PriorSc_(PriorSc, K, D);
	SafeArray<const numtype> PriorA1_(PriorA1, K, D);
	SafeArray<const numtype> PriorA2_(PriorA2, K, D);

	SafeArray<const numtype> Data_(Data, N, DNMaxRepl);
	SafeArray<const numtype> NRepl_(NRepl, N, D);

	SafeArray<numtype> LogPKGivenY_(LogPKGivenY, N, K);

	numtype *da = new numtype[N*DNMaxRepl];
	numtype *LogPKGivenYCurr = new numtype[K];
	numtype *LogPYGivenK = new numtype[K];
	int i,j,r,k;
	numtype x, tmp, v1, v2, v3, v4;
	for (i = 1; i <= N; i++)
	{
		memset(LogPKGivenYCurr, 0, sizeof(numtype)*K);

		int NDa = 0;
		for (j = 1; j <= D; j++)
		{
			for (r = 1; r <= NRepl_(i,j); r++)
			{
				x = Data_(i, r + (j-1)*NMaxRepl);
				if (x != -Inf)
				{
					da[NDa++] = x;
				}
			}
		}
		for (k = 1; k <= K; k++)
		{
			GetLogPYNormalIGamma1Dim(&tmp, &v1, &v2, &v3, &v4, da, PriorMu_(k,1), PriorSc_(k,1), PriorA1_(k,1), PriorA2_(k,1), NDa);
			LogPYGivenK[k-1] = tmp  ;
		}
		InvertProb (LogPY + i - 1, LogPKGivenYCurr, LogPYGivenK, PK, K);
		for (k = 1; k <= K; k++)
			LogPKGivenY_(i,k) = LogPKGivenYCurr[k-1];
	}
	delete[] da;
	delete[] LogPKGivenYCurr;
	delete[] LogPYGivenK;
}
void GetLogPYNormalIGammaReplClust(
						  numtype *LogPY, 
						  numtype *PostMu, 
						  numtype *PostSc, 
						  numtype *PostA1, 
						  numtype *PostA2, 
						  numtype *PostZ, 
				    const numtype *Data, 
					const numtype *NRepl,
					const numtype *PriorMu, 
					const numtype *PriorSc, 
					const numtype *PriorA1, 
					const numtype *PriorA2, 
					const numtype *Z,
					const int K, 
					const int N, 
					const int D,
					const int DNMaxRepl)
{
	int NMaxRepl = DNMaxRepl / D;

	SafeArray<numtype> PostMu_(PostMu, K, D);
	SafeArray<numtype> PostSc_(PostSc, K, D);
	SafeArray<numtype> PostA1_(PostA1, K, D);
	SafeArray<numtype> PostA2_(PostA2, K, D);

	SafeArray<const numtype> PriorMu_(PriorMu, K, D);
	SafeArray<const numtype> PriorSc_(PriorSc, K, D);
	SafeArray<const numtype> PriorA1_(PriorA1, K, D);
	SafeArray<const numtype> PriorA2_(PriorA2, K, D);

	SafeArray<const numtype> Data_(Data, N, DNMaxRepl);
	SafeArray<const numtype> NRepl_(NRepl, N, D);

	int j, k, r, i;
	numtype v1,v2, v3, v4, x, tmp;
	numtype *da = new numtype[N*NMaxRepl];
	for (j = 1; j <= D; j++)
	{
		for (k = 1; k <= K; k++)
		{
			int NDa = 0;
			for (i = 1; i <= N; i++)
			{
				if (Z[i-1] == k)
				{
					for (r = 1; r <= NRepl_(i,j); r++)
					{
						x = Data_(i, r + (j-1)*NMaxRepl);
						if (x != -Inf)
						{
							da[NDa++] = x;
						}
					}
				}
			}
			GetLogPYNormalIGamma1Dim(&tmp, &v1, &v2, &v3, &v4, da, PriorMu_(k,j), PriorSc_(k,j), PriorA1_(k,j), PriorA2_(k,j), NDa);
			LogPY[k-1] += tmp;
			PostMu_(k,j) = v1;
			PostSc_(k,j) = v2;
			PostA1_(k,j) = v3;
			PostA2_(k,j) = v4;
		}
	}
	delete[] da;
}

void GetLogPYNormalIGammaReplSClust(
						  numtype *LogPY, 
						  numtype *PostMu, 
						  numtype *PostSc, 
						  numtype *PostA1, 
						  numtype *PostA2, 
						  numtype *PostZ, 
				    const numtype *Data, 
					const numtype *NRepl,
					const numtype *PriorMu, 
					const numtype *PriorSc, 
					const numtype *PriorA1, 
					const numtype *PriorA2, 
					const numtype *Z,
					const int K, 
					const int N, 
					const int D,
					const int DNMaxRepl)
{
	int NMaxRepl = DNMaxRepl / D;

	SafeArray<numtype> PostMu_(PostMu, K, D);
	SafeArray<numtype> PostSc_(PostSc, K, D);
	SafeArray<numtype> PostA1_(PostA1, K, D);
	SafeArray<numtype> PostA2_(PostA2, K, D);

	SafeArray<const numtype> PriorMu_(PriorMu, K, D);
	SafeArray<const numtype> PriorSc_(PriorSc, K, D);
	SafeArray<const numtype> PriorA1_(PriorA1, K, D);
	SafeArray<const numtype> PriorA2_(PriorA2, K, D);

	SafeArray<const numtype> Data_(Data, N, DNMaxRepl);
	SafeArray<const numtype> NRepl_(NRepl, N, D);

	int i,j,k,r;
	numtype v1, v2, v3, v4, x, tmp;
	numtype *da = new numtype[N*DNMaxRepl];
	for (k = 1; k <= K; k++)
	{
		int NDa = 0;
		for (i = 1; i <= N; i++)
		{
			if (Z[i-1] == k)
			{
				for (j = 1; j <= D; j++)
				{
					for (r = 1; r <= NRepl_(i,j); r++)
					{
						x = Data_(i, r + (j-1)*NMaxRepl);
						if (x != -Inf)
						{
							da[NDa++] = x;
						}
					}
				}
			}
		}

		GetLogPYNormalIGamma1Dim(&tmp, &v1, &v2, &v3, &v4, da, PriorMu_(k,1), PriorSc_(k,1), PriorA1_(k,1), PriorA2_(k,1), NDa);
		LogPY[k-1] += tmp;
		for (j = 1; j <= D; j++)
		{
			PostMu_(k,j) = v1;
			PostSc_(k,j) = v2;
			PostA1_(k,j) = v3;
			PostA2_(k,j) = v4;
		}
	}
	delete[] da;
}
