#ifndef SAFE_ARRAY_H_
#define SAFE_ARRAY_H_


#include <vector>
#include <cassert>

template <class T>

class OneDIterator {

    public:

//        typedef forward_iterator_tag iterator_category;

        typedef T                    value_type;

        typedef ptrdiff_t            difference_type;

        typedef T*                   pointer;

        typedef T&                   reference;



        OneDIterator(T *p, int stride):p_(p),stride_(stride){

            assert(p != NULL);

        }

        OneDIterator(const OneDIterator& x):p_(x.p_),stride_(x.stride_){}

        OneDIterator& operator=(const OneDIterator& x) {

            if (this == &x) return *this;

            p_ = x.p_;

            stride_ = x.stride_;

            return *this;

        }

        bool operator==(const OneDIterator& x) {

            return (p_ == x.p_);

        }

        bool operator!=(const OneDIterator& x) {

            return (p_ != x.p_);

        }

        T& operator*() {

            return *p_;

        }

        OneDIterator& operator++() {   // ++i

            p_ += stride_;

            return *this;

        }

        OneDIterator operator++(int) { // i++

            OneDIterator tmp(*this);

            p_ += stride_;

            return tmp;

        }

    private:

        T *p_;

        int stride_;

};



template <class T>

class SafeArray {

 public:

    SafeArray(T *a, int n1): a_(a), a2_(a-1), n_dim_(1), n1_(n1), n2_(0), n3_(0), size_(n1) {

       assert(a != NULL);

       assert(n1 > 0);

    }

    SafeArray(T *a, int n1, int n2): a_(a), 

#ifndef MEX

    a2_(a-n2-1),

#else

    a2_(a-n1-1),

#endif

       n_dim_(2), n1_(n1), n2_(n2), n3_(0), size_(n1*n2) {

       assert(a != NULL);

       assert(n1 > 0);

       assert(n2 > 0);

    }

    SafeArray(T *a, int n1, int n2, int n3): a_(a), 

#ifndef MEX

a2_(a-n2*n3-n3-1),

#else

a2_(a-n2*n1-n1-1),

#endif

              n_dim_(3), n1_(n1), n2_(n2), n3_(n3),size_(n1*n2*n3) {

       assert(a != NULL);

       assert(n1 > 0);

       assert(n2 > 0);

       assert(n3 > 0);

    }

    void Resize(int n1) {

	   a2_ = a_-1; 
	   n_dim_ = 1;
	   n1_ = n1;  
	   n2_ = 0; 
	   n3_ = 0;
	   size_ = n1;

       assert(n1 > 0);
    };

    void Resize(int n1, int n2){

#ifndef MEX

    a2_ = (a_-n2-1);

#else

    a2_ = (a_-n1-1);

#endif

       n_dim_ = (2);
	   n1_ = (n1);
	   n2_ = (n2);
	   n3_ = (0); 
	   size_ = (n1*n2);

       assert(n1 > 0);
       assert(n2 > 0);
    };

    void Resize(int n1, int n2, int n3)
	{

#ifndef MEX

a2_ = (a_-n2*n3-n3-1);

#else

a2_ = (a_-n2*n1-n1-1);

#endif

       n_dim_ = (3);
	   n1_ = (n1);
	   n2_ = (n2);
	   n3_ = (n3);
	   size_ = (n1*n2*n3);

       assert(n1 > 0);
       assert(n2 > 0);
       assert(n3 > 0);
    }

    T &operator()(int i) {

       assert(n_dim_ == 1);

       assert(i >= 1);

       assert(i <= n1_);

       return a2_[i];

    };

    T &operator()(int i, int j) {

       assert(n_dim_ == 2);

       assert(i >= 1);

       assert(i <= n1_);

       assert(j >= 1);

       assert(j <= n2_);

#ifndef MEX

       return a2_[i*n2_ + j];

#else

       return a2_[j*n1_ + i];

#endif

    }

    T &operator()(int i, int j, int k) {

       assert(n_dim_ == 3);

       assert(i >= 1);

       assert(i <= n1_);

       assert(j >= 1);

       assert(j <= n2_);

       assert(k >= 1);

       assert(k <= n3_);

#ifndef MEX

       return a2_[(i*n2_ + j)*n3_ + k];

#else

       return a2_[i + (j + k*n2_)*n1_];

#endif

    }

    T &c_index(int i) {

       assert(n_dim_ == 1);

       assert(i >= 0);

       assert(i < n1_);

       return a_[i];

    }

    T &c_index(int i, int j) {

       assert(n_dim_ == 2);

       assert(i >= 0);

       assert(i < n1_);

       assert(j >= 0);

       assert(j < n2_);

#ifndef MEX

       return a_[i*n2_ + j];

#else

       return a_[j*n1_ + i];

#endif

    }

    T &c_index(int i, int j, int k) {

       assert(n_dim_ == 3);

       assert(i >= 0);

       assert(i < n1_);

       assert(j >= 0);

       assert(j < n2_);

       assert(k >= 0);

       assert(k < n3_);

#ifndef MEX

       return a_[(i*n2_ + j)*n3_ + k];

#else

       return a_[i + (j + k*n2_)*n1_];

#endif

    }



    OneDIterator<T> row_begin(int r) {

        assert(n_dim_==2);

        assert(r >= 1);

        assert(r <= n1_);

#ifndef MEX

        return OneDIterator<T>(a_ + (r-1) * n2_, 1);

#else

        return OneDIterator<T>(a_ + (r-1), n1_);

#endif

    }

    OneDIterator<T> column_begin(int c) {

        assert(n_dim_==2);

        assert(c >= 1);

        assert(c <= n2_);

#ifndef MEX

        return OneDIterator<T>(a_ + (c-1), n2_);

#else

        return OneDIterator<T>(a_ + (c-1) * n1_, 1);

#endif

    }

    OneDIterator<T> row_end(int r) {

        assert(n_dim_==2);

        assert(r >= 1);

        assert(r <= n1_);

#ifndef MEX

        return OneDIterator<T>(a_ + r * n2_, 1);

#else

        return OneDIterator<T>(a_ + size_ + r-1, n1_);

#endif

    }

    OneDIterator<T> column_end(int c) {

        assert(n_dim_==2);

        assert(c >= 1);

        assert(c <= n2_);

#ifndef MEX

        return OneDIterator<T>(a_ + size_ + c-1, n2_);

#else

        return OneDIterator<T>(a_ + c * n1_, 1);

#endif

    }



    T* begin() { return a_; }

    T* end() {

                assert(n_dim_ >= 1);

                assert(n_dim_ <= 3);

                return a_ + size_;

             }

  private:

    T* const a_;     /* pointer to data storage */

    T* const a2_;    /* offseted so the indices start at 1 */

    const int n_dim_;

    const int n1_, n2_, n3_;

    const int size_;

};

#endif

