#ifndef DEFS_H_
#define DEFS_H_

#ifdef __cplusplus
extern "C" {
#endif

#define DEBUG_WARN

typedef double numtype;

#define		DISTRIB_NORMAL		1
#define		DISTRIB_LOGNORMAL	2
#define		DISTRIB_UNIFORM		3
#define		DISTRIB_LAPLACE		4

#define Inf (HUGE_VAL)
#define PI	(3.1415926535897)

#define FREEZE_MASK_MU (0x0001)
#define FREEZE_MASK_REPLVAR (0x002)
#define FREEZE_MASK_ALPHA (0x0004)
#define FREEZE_MASK_MU0 (0x0010)
#define FREEZE_MASK_MG (0x0020)
#define FREEZE_MASK_INJ (0x0040)
#define FREEZE_MASK_Z (0x0080)
#define FREEZE_MASK_KFACTOR (0x0100)
#define FREEZE_MASK_YNORM (0x0200)

#define MODEL_NOMU0_MASK (1)
#define MODEL_MU0_MASK	(2)
#define MODEL_SCALED_MASK (8)
#define MODEL_KFACTOR_INSIDE_MASK (16)
#define MODEL_KFACTOR_INSIDE_RV_MASK (32)
#define MODEL_MU0_DEP_MASK	(64)
#define MODEL_REPLVAR_DEP_MASK (128)
#define MODEL_VERSION_COVOFFS (256)
#define MODEL_LEARN_IGAMMA_DF (512)

#define MODEL_NORMALIGAMMA_NHC (1)
#define MODEL_NORMALIGAMMA_INDEP_NHC (2)

#define COMBINED_INIT_SIZE_KHC_1 (6)
#define COMBINED_INIT_SIZE_KHC_2 (DNMaxRepl)

#define COMBINED_INIT_SIZE_KINSIDE_1 (5)
#define COMBINED_INIT_SIZE_KINSIDE_2 (D)

#define GIBBS_KINSIDEHC_POSTERIOR_DIM (5+D+D+T2+K+1)
#define GIBBS_KINSIDEHC_GENERIC_POSTERIOR_DIM (5+D+D+T+K+1)
#define GIBBS_POSTERIOR_DIM (4 + 2*D + T2 + K + 1)

#define COMBINED_PRIORS_GENERIC_PER_SIZE1 (3)
#define COMBINED_PRIORS_GENERIC_BG_SIZE1 (2)



class Distrib
{
public:
	Distrib (int type_, numtype mean_, numtype sigmasq_) : type(type_), mean(mean_), sigmasq(sigmasq_)
	{
	};

	void LogDensity (numtype *Y, const numtype *X, int m, int n) const;
	//% input  X(m,n)
	//% output Y(m,n)
	void DLogDensity (numtype *Y, const numtype *X, int m, int n) const;
	//% input  X(m,n)
	//% output Y(m,n)
	void sample (numtype *y, const int n) const;
	//% output y (1,n)

private:
	int type;
	numtype mean;
	numtype sigmasq;
};

class PriorInfo
{
public:
	Distrib *W;
	Distrib *A;
	Distrib *L;

	PriorInfo (Distrib *W_, Distrib *A_, Distrib *L_) : W(W_), A(A_), L(L_){};
};

#ifdef __cplusplus
}
#endif

#endif
