#include <memory.h>
#include <math.h>
#include "defs.h"
#include "utils.h"
#include "quickSort.h"
#include "median.h"


/*
 * Algorithm from N. Wirth's book, implementation by N. Devillard.
 * This code in public domain.
 */


#define ELEM_SWAP(a,b) { register numtype t=(a);(a)=(b);(b)=t; }

/*---------------------------------------------------------------------------
   Function :   kth_smallest()
   In       :   array of elements, # of elements in the array, rank k
   Out      :   one element
   Job      :   find the kth smallest element in the array
   Notice   :   use the median() macro defined below to get the median. 

                Reference:

                  Author: Wirth, Niklaus 
                   Title: Algorithms + data structures = programs 
               Publisher: Englewood Cliffs: Prentice-Hall, 1976 
    Physical description: 366 p. 
                  Series: Prentice-Hall Series in Automatic Computation 

 ---------------------------------------------------------------------------*/


numtype kth_smallest(numtype a[], int n, int k)
{
    register int i,j,l,m ;
    register numtype x ;

    l=0 ; m=n-1 ;
    while (l<m) {
        x=a[k] ;
        i=l ;
        j=m ;
        do {
            while (a[i]<x) i++ ;
            while (x<a[j]) j-- ;
            if (i<=j) {
                ELEM_SWAP(a[i],a[j]) ;
                i++ ; j-- ;
            }
        } while (i<=j) ;
        if (j<k) l=i ;
        if (k<i) m=j ;
    }
    return a[k] ;
}


#define median(a,n) kth_smallest(a,n,(((n)&1)?((n)/2):(((n)/2)-1)))

numtype TMedian::Eval(const numtype *v, const int len)
	{
		if (arLen < len)
		{
			delete[] sortArray; 
			sortArray = new numtype[len];
			arLen = len;
		}
		memcpy(sortArray, v, sizeof(numtype)*len);
		quickSort(sortArray, len);

		return len & 1 ? sortArray[len/2] : 0.5*(sortArray[(len/2)-1] + sortArray[len/2]);
	}

numtype TMedian::Eval(const int len)
	{
		if (arLen < len)
		{
			return -Inf;
		}
		quickSort(sortArray, len);

		return len & 1 ? sortArray[len/2] : 0.5*(sortArray[(len/2)-1] + sortArray[len/2]);
	}
		
numtype TMedian::EvalFast(const numtype *v, const int len)
	{
		if (arLen < len)
		{
			delete[] sortArray; 
			sortArray = new numtype[len];
			arLen = len;
		}
		memcpy(sortArray, v, sizeof(numtype)*len);
		return median(sortArray, len);
	}

numtype TMedian::EvalFast(const int len)
	{
		if (arLen < len)
		{
			return -Inf;
		}

		return median(sortArray, len);
	}
		
void TMedian::Set(const numtype v, const int i)
	{
		if (arLen < i)
		{
			numtype *backup = new numtype [arLen];
			memcpy(backup, sortArray, sizeof(numtype)*arLen);

			delete[] sortArray; 

			sortArray = new numtype[2*i];
			memcpy(sortArray, backup, sizeof(numtype)*arLen);

			arLen = 2*i;
		}
		sortArray[i-1] = v;
	}


