
#include <math.h>
#include <time.h>
#include "SafeArray.h"
#include "defs.h"
#include "utils.h"
#include "mex.h"
#include "quickSort.h"
#include <limits.h>

#define sprintf_s snprintf

//#define Inf (HUGE_VAL)
//#define PI	(3.1415926535897)
#define GAMRND_MAX_REJECT (1000)

#include "mf_updpost.h"
#include "ran/randlib/randlib.h"
#include "median.h"

// -0.5*log(2*pi) = -0.91893853320467267
#define _HALF_LOG_2_PI_ (-0.91893853320467267)

TMedian Med;

void RandlibSetSeed(int seed1, int seed2)
{
	setall(seed1,seed2);
}


numtype add_in_log (numtype x, numtype y) {
    numtype log_a;
    numtype log_b;

    if (x > y) {
        log_a = x;
        log_b = y;
    } else {
        log_a = y;
        log_b = x;
    }

    if (log_b == -Inf)
        return log_a;

    return (log_a + log (1.0 + exp (log_b - log_a)));
}

/* exp(-Inf) == NaN for some versions of libc, so let's be safe! */
numtype safe_exp (numtype x) {
    if (x == -Inf)
        return 0.;
    return exp (x);
}

void AddLogScale (numtype *y, const numtype *x, const int iFirst, const int iLast, const int N) {

	SafeArray<const numtype> x_(x, N);

	int i;
	numtype mValue = -Inf;

	//% find max value
	for (i = iFirst; i <= iLast; i++)
	{
		if (x_(i) > mValue)
		{
			mValue = x_(i);
		}
	}

	numtype s = 0;
	for (i = iFirst; i <= iLast; i++)
	{
		s += safe_exp(x_(i)-mValue);
	}
	
	*y = mValue + log(s);
}

void MeanLogScale (numtype *y, const numtype *x, const int iFirst, const int iLast, const int N) 
{
	SafeArray<const numtype> x_(x, N);

	int i;
	numtype mValue = -Inf;

	//% find max value
	for (i = iFirst; i <= iLast; i++)
	{
		if (x_(i) > mValue)
		{
			mValue = x_(i);
		}
	}

	numtype s = 0;
	for (i = iFirst; i <= iLast; i++)
	{
		s += safe_exp(x_(i)-mValue);
	}
	
	*y = mValue + log(s) - log(numtype(iLast - iFirst + 1));
}

// calculate p(z|X) and log(p(X)) given log(p(X|z)) and P(z)
void InvertProb (numtype *LogPX, numtype *LogPZGivenX, const numtype *LogPXGivenZ, const numtype *PZ, const int D) 
{
	SafeArray<const numtype> LogPXGivenZ_(LogPXGivenZ, D);
	SafeArray<const numtype> PZ_(PZ, D);

	SafeArray<numtype> LogPZGivenX_(LogPZGivenX, D);

	int i;
	numtype mValue = -Inf;

	//% find max value
	for (i = 1; i <= D; i++)
		mValue = mValue < LogPXGivenZ_(i) ? LogPXGivenZ_(i) : mValue;

	numtype norm = 0;
	for (i = 1; i <= D; i++)
	{
		if (LogPXGivenZ_(i) == -Inf)
			LogPZGivenX_(i) = 0;
		else
			LogPZGivenX_(i) = safe_exp(LogPXGivenZ_(i)-mValue)*PZ_(i);

		norm += LogPZGivenX_(i);
	}
	*LogPX = mValue + log(norm);
	
	for (i = 1; i <= D; i++)
	{
		LogPZGivenX_(i) /= norm;
	}
}

// calculate p(z|X) and log(p(X)) given log(p(X|z)) and P(z) for multiple observations 
void InvertProb2 (numtype *LogPX, numtype *LogPZGivenX, const numtype *LogPXGivenZ, const numtype *PZ, const int D, const int N) 
{
	SafeArray<const numtype> LogPXGivenZ_(LogPXGivenZ, N, D);
	SafeArray<const numtype> PZ_(PZ, D);

	SafeArray<numtype> LogPZGivenX_(LogPZGivenX, N, D);
	SafeArray<numtype> LogPX_(LogPX, N);

	int i, n;
	numtype mValue;

	for (n = 1; n <= N; n++)
	{
		//% find max value
		mValue = -Inf;
		for (i = 1; i <= D; i++)
			mValue = mValue < LogPXGivenZ_(n,i) ? LogPXGivenZ_(n,i) : mValue;

		numtype norm = 0;
		for (i = 1; i <= D; i++)
		{
			if (LogPXGivenZ_(n,i) == -Inf)
				LogPZGivenX_(n,i) = 0;
			else
				LogPZGivenX_(n,i) = safe_exp(LogPXGivenZ_(n,i)-mValue)*PZ_(i);

			norm += LogPZGivenX_(n,i);
		}
		LogPX_(n) = mValue + log(norm);
		
		for (i = 1; i <= D; i++)
		{
			LogPZGivenX_(n,i) /= norm;
		}
	}
}

///////////// Knuths algorithm ////////////////////////////////////////////
void knuthPerm(register numtype * input, int n)
{ 
  register double nDouble = (double) n;
  register numtype temp;
  register numtype *beyond = input + n - 1;
  register numtype *other;
 
  register double ratio;

  for (;  input < beyond;  input++, nDouble--) {
	ratio = (nDouble-1)/RAND_MAX;
    other      = input + int(ratio*rand());
    temp       = *input;
    *input     = *other;
    *other     = temp;
  }
}

// sample n times from the distribution 
void Distrib::sample (numtype *Y, int n) const
{
    SafeArray<numtype> Y_(Y,n);

	int i;

	if (type == DISTRIB_NORMAL)
		// sample normal distribution
		for (i = 1; i <= n; i++)
			Y_(i) = 0;
		
	else if (type == DISTRIB_LOGNORMAL)
		// sample lognormal distribution
		for (i = 1; i <= n; i++)
			Y_(i) = 0;
		
	else if (type == DISTRIB_LAPLACE)
	{
		// DAC : do smth or oautput an error
	}
	else 
	{
		// DAC : output an error!
	}
}

// logarithm of the density function at X
void Distrib::LogDensity (numtype *Y, const numtype *X, int m, int n) const
{
	int i, j;

	if (type == DISTRIB_NORMAL)
		GetLogP1Dim (Y, X, mean, sigmasq, 1/sigmasq, m, n);
	else if (type == DISTRIB_LOGNORMAL)
	{
		numtype *logX = new numtype[m*n];
		SafeArray<const numtype> X_(X,m,n);
		SafeArray<numtype> logX_(logX,m,n);
		
		for (i = 1; i <= m; i++)
			for (j = 1; j <= n; j++)
				logX_(i,j) = log(X_(i,j));

		GetP1Dim (Y, logX, mean, sigmasq, 1/sigmasq, m, n);

		delete[] logX;
	}
	else if (type == DISTRIB_LAPLACE)
	{
		// DAC : do smth or oautput an error
	}
	else 
	{
		// DAC : output an error!
	}
}

// Derivative of the logarithm of the density function at X
void Distrib::DLogDensity (numtype *Y, const numtype *X, int m, int n) const
{
    SafeArray<const numtype> X_(X,m,n);
    SafeArray<numtype> Y_(Y,m,n);

	int i,j;

	if (type == DISTRIB_NORMAL)
	{
		for (i = 1; i <= m; i++)
			for (j = 1; j <= n; j++)
				Y_(i,j) = -(X_(i,j)-mean) / sigmasq;
	}
	else if (type == DISTRIB_LOGNORMAL)
	{
		numtype *logX = new numtype[m*n];
		SafeArray<const numtype> X_(X,m,n);
		SafeArray<numtype> logX_(logX,m,n);
		
		for (i = 1; i <= m; i++)
			for (j = 1; j <= n; j++)
				logX_(i,j) = log(X_(i,j));

		GetP1Dim (Y, logX, mean, sigmasq, 1/sigmasq, m, n);

		for (i = 1; i <= m; i++)
			for (j = 1; j <= n; j++)
				Y_(i,j) = -Y_(i,j) * (logX_(i,j) - mean) / sigmasq;

		delete[] logX;
			
	}
	else if (type == DISTRIB_LAPLACE)
	{
		// DAC: do smth or output an error!
	}
	else
	{
		// DAC : output an error!
	}
}

void sample (numtype *y)
{
}


// Calculates the derivative of the log of 1Dim Gauusian density at matrix of inputs X
void GetDLogP1Dim(numtype *y, 
				const numtype *X, 
				const numtype Mean, 
				const numtype Det, 
				const numtype InvCov, 
				const int m, 
				const int n
				)
{
    SafeArray<const numtype> X_(X,m,n);
    SafeArray<numtype> Y_(y,m,n);

	int i,j;

	for (i = 1; i <=m; i++)
	{
		for (j = 1; j <= n; j++)
		{
			Y_(i,j) = -(X_(i,j) - Mean) * InvCov ;
		}
	}
}

// Calculates the log of the 1Dim Gauusian density at matrix of inputs X
void GetLogP1Dim(numtype *y, 
				const numtype *X, 
				const numtype Mean, 
				const numtype Det, 
				const numtype InvCov, 
				const int m, 
				const int n
				)
{
    SafeArray<const numtype> X_(X,m,n);
    SafeArray<numtype> Y_(y,m,n);

	int i,j;
	numtype diff, C;

	// C =  1/ (((2.0*PI)^(1/2)) * sqrt(Det) );
	// -0.5*log(2*pi) = -0.91893853320467267
	C = _HALF_LOG_2_PI_ - 0.5*log(Det);
	
	for (i = 1; i <=m; i++)
	{
		for (j = 1; j <= n; j++)
		{
			diff = X_(i,j) - Mean;

			Y_(i,j) = C - 0.5 * diff * InvCov * diff;
		}
	}
}

numtype GetLogP1Dim1(const numtype X, const numtype Mean, const numtype InvCov, const numtype LogInvCov)
{
	// C =  1/ (((2.0*PI)^(1/2)) * sqrt(Det) );
	// -0.5*log(2*pi) = -0.91893853320467267
	
	numtype diff= X - Mean;
	return _HALF_LOG_2_PI_ + 0.5*(LogInvCov - diff * InvCov * diff);
}

// Calculates the log of the Gauusian density with diagonal cov matrix CovDiag at matrix of inputs X
void GetLogPDiag(numtype *y, 
				const numtype *X, 
				const numtype *Mean, 
				const numtype *CovDiag, 
				const int D, 
				const int N
				)
{
    SafeArray<const numtype> X_			(X,D,N);
    SafeArray<const numtype> Mean_		(Mean,D);
    SafeArray<const numtype> CovDiag_	(CovDiag,D);
    SafeArray<numtype>		 Y_			(y,N);

	int i,j;
	numtype diff, C;

	// C =  1/ (((2.0*PI)^(1/2)) * sqrt(Det) );
	C = 0;
	for (j = 1; j <= D; j++)
	{
		C += log(CovDiag_(j));
	}
	C *= -0.5;
	C += D*_HALF_LOG_2_PI_;

	for (i = 1; i <= N; i++)
	{
		Y_(i) = 0;
		for (j = 1; j <= D; j++)
		{
			diff = X_(j,i) - Mean_(j);

			Y_(i) += (diff*diff) / CovDiag_(j);
		}
		Y_(i) = C - 0.5 * Y_(i);
	}
}

// Calculates the log of the Gauusian density with diagonal cov matrix CovDiag at matrix of inputs X
void GetLogPDiagInv(numtype *y, 
				const numtype *X, 
				const numtype *Mean, 
				const numtype *InvCovDiag, 
				const int D, 
				const int N
				)
{
    SafeArray<const numtype> X_			(X,D,N);
    SafeArray<const numtype> Mean_		(Mean,D);
    SafeArray<const numtype> InvCovDiag_(InvCovDiag,D);
    SafeArray<numtype>		 Y_			(y,N);

	int i,j;
	numtype diff, C;

	// C =  1/ (((2.0*PI)^(1/2)) * sqrt(Det) );
	C = 0;
	for (j = 1; j <= D; j++)
	{
		C -= log(InvCovDiag_(j));
	}
	C *= -0.5;
	C += D*_HALF_LOG_2_PI_;

	for (i = 1; i <= N; i++)
	{
		Y_(i) = 0;
		for (j = 1; j <= D; j++)
		{
			diff = X_(j,i) - Mean_(j);

			Y_(i) += (diff*diff) * InvCovDiag_(j);
		}
		Y_(i) = C - 0.5 * Y_(i);
	}
}

// Calculates the 1Dim Gauusian density at matrix of inputs X
void GetP1Dim(numtype *y, 
				const numtype *X, 
				const numtype Mean, 
				const numtype Det, 
				const numtype InvCov, 
				const int m, 
				const int n
				)
{
    SafeArray<const numtype> X_(X,m,n);
    SafeArray<numtype> Y_(y,m,n);

	int i,j;
	numtype diff, C;

	// C =  1/ (((2.0*PI)^(1/2)) * sqrt(Det) );
	C = 1.0 / sqrt(2.0*PI*Det);
	
	for (i = 1; i <=m; i++)
	{
		for (j = 1; j <= n; j++)
		{
			diff = X_(i,j) - Mean;

			Y_(i,j) = C * exp ( - 0.5 * diff * InvCov * diff);
		}
	}
}

// Calculates the log of the t-distribution pdf with nu df
void GetLogPT(numtype *Y, const numtype *X, const numtype nu, const numtype mu, const numtype ssq, const int m, const int n)
{
    SafeArray<const numtype> X_			(X,m,n);
    SafeArray<numtype>		 Y_			(Y,m,n);
	
	// 
	int i,j;
	numtype tmp, C;

	numtype g1, g2;
	lgamma_c(&g1, 0.5*(nu+1));
	lgamma_c(&g2, 0.5*nu);
	
	C = g1 - g2 - 0.5*log(PI*nu*ssq);

	for (i = 1; i <=m; i++)
	{
		for (j = 1; j <= n; j++)
		{
			tmp = X_(i,j);
			Y_(i,j) =  C - 0.5*(nu + 1) * log(1 + (tmp-mu)*(tmp-mu)/ (nu*ssq));
		}
	}
}

// Calculates the log of the t-distribution pdf with nu df
numtype GetLogPT1(const numtype X, const numtype nu, const numtype mu, const numtype ssq)
{
	// 
	numtype C;

	numtype g1, g2;
	lgamma_c(&g1, 0.5*(nu+1));
	lgamma_c(&g2, 0.5*nu);
	
	C = g1 - g2 - 0.5*log(PI*nu*ssq);

	return C - 0.5*(nu + 1) * log(1 + (X-mu)*(X-mu) / (nu*ssq));
}

// Calculates the sigmoid function g = 1 / (1+exp(-x)) for a matrix of inputs x
void sigmoid(numtype *Y, const numtype *X, const int m, const int n)
{
    SafeArray<const numtype> X_(X,m,n);
    SafeArray<numtype> Y_(Y,m,n);

	int i,j;

	for (i = 1; i <=m; i++)
	{
		for (j = 1; j <= n; j++)
		{
			Y_(i,j) = 1.0 / (1 + exp(-X_(i,j)));
		}
	}
}

// Calculates the derivative of the sigmoid function g = 1 / (1+exp(-x)) for a matrix of inputs x
void dsigmoid(numtype *Y, const numtype *X, const int m, const int n)
{
    SafeArray<const numtype> X_(X,m,n);
    SafeArray<numtype> Y_(Y,m,n);

	int i,j;
	numtype tmp;

	
	for (i = 1; i <=m; i++)
	{
		for (j = 1; j <= n; j++)
		{
			tmp = exp(-X_(i,j));

			Y_(i,j) = tmp / ((1 + tmp)*(1+tmp));

		}
	}
}


// return a linear index given three-way index and dimensions
int Sub2Ind (int i, int j, int k, int N1, int N2, int N3)
{
	return i + ((j-1) + (k-1)*N2)*N1;
}

// Sample N values from  a discrete distribution given by p
void sample (numtype *v, const numtype *p, const int N, const int D)
{

    SafeArray<const numtype> p_(p,D);
    SafeArray<numtype> v_(v,N);

	int i,j, idx;
	numtype r, sum;

	for (i = 1; i <= N; i++)
	{
		r = ranf();

		idx = -1;
		sum = 0.0;

		j = 1;
		while (idx < 0 && j <= D)
		{
			sum += p_(j);

			if (r <= sum)
				idx = j;

			j++;
		}
	
		if (idx < 0) 
			idx = D;

		v_(i) = idx;
	}
}


void GetLogPIGamma (numtype *p, const numtype *x, const numtype PriorA, const numtype PriorB, const int M, const int N)
{
	SafeArray<numtype> p_(p, M, N);
	SafeArray<const numtype> x_(x, M, N);

	numtype alpha = PriorA;
	numtype beta = 1./PriorB;

	numtype GammaLnAlpha;
	lgamma_c(&GammaLnAlpha, alpha);

	numtype AlphaLogBeta = alpha*log(beta);

	int i,j;
	for (i = 1; i <= M; i++)
	{
		for (j = 1; j <= N; j++)
		{
			p_(i,j) = AlphaLogBeta  - GammaLnAlpha - (alpha + 1)*log(x_(i,j)) - beta/x_(i,j);
		}
	}
}

numtype GetLogPIGamma1 (const numtype x, const numtype PriorA, const numtype PriorB)
{
	numtype alpha = PriorA;

	numtype GammaLnAlpha;
	lgamma_c(&GammaLnAlpha, alpha);

	numtype AlphaLogPriorB = -alpha*log(PriorB);

	return AlphaLogPriorB  - GammaLnAlpha - (alpha + 1)*log(x) - 1./(PriorB*x);
}

void IGammaStat (numtype *m, numtype *v, const numtype PriorA, const numtype PriorB)
{
	if (PriorA > 1)
		*m = 1/((PriorA-1)*PriorB);
	else 
		*m = -Inf;
	
	if (PriorA > 2)
		*v = 1/((PriorA-1)*(PriorA-1)*(PriorA-2)*PriorB*PriorB);
	else
		*v = -Inf;
}


void GetLogPGamma (numtype *p, const numtype *x, const numtype PriorA, const numtype PriorB, const int M, const int N)
{
	SafeArray<numtype> p_(p, M, N);
	SafeArray<const numtype> x_(x, M, N);

	numtype alpha = PriorA;
	numtype beta = 1./PriorB;

	numtype GammaLnAlpha;
	lgamma_c(&GammaLnAlpha, alpha);

	numtype AlphaLogBeta = alpha*log(beta);

	int i,j;
	for (i = 1; i <= M; i++)
	{
		for (j = 1; j <= N; j++)
		{
			p_(i,j) = AlphaLogBeta  - GammaLnAlpha - (alpha - 1)*log(x_(i,j)) - beta*x_(i,j);
		}
	}
}

numtype GetLogPGamma1 (const numtype x, const numtype PriorA, const numtype PriorB)
{

	numtype alpha = PriorA;
	numtype beta = 1./PriorB;

	numtype GammaLnAlpha;
	lgamma_c(&GammaLnAlpha, alpha);

	numtype AlphaLogBeta = alpha*log(beta);

	return (AlphaLogBeta  - GammaLnAlpha - (alpha - 1)*log(x) - beta*x);
}

// Generate a sample from a gamma distribution
void gamrnd_c(numtype *X, numtype a, numtype b, int M, int N)
{
//  GAMRND Random matrices from gamma distribution.
//   R = GAMRND(A,B, M, N) returns a matrix of random numbers chosen   
//   from the gamma distribution with parameters A and B.

//   GAMRND uses a rejection or an inversion method depending on the
//   value of A. 

//%   References:
//%      [1]  L. Devroye, "Non-Uniform Random Variate Generation", 
//%      Springer-Verlag, 1986

//%   B.A. Jones 2-1-93
//%   Copyright 1993-2000 The MathWorks, Inc. 
//%   $Revision: 2.11 $  $Date: 2000/05/26 18:52:53 $

    SafeArray<numtype> X_(X,M,N);
    
	int m, n, accept, counter;
	numtype c, d, bb, x, e, z, u, v, w, y, r;

	
	//% If a == 1, then gamma is exponential. (Devroye, page 405).
	if (a == 1) 
	{
		for (m = 1; m <= M; m++)
		{
			for (n = 1; n <= N; n++)
			{
				X_(m,n) = -b * log(ranf());
			}
		}
	} 
	else if (a < 1 && a > 0)
	{
	//% (Devroye, page 418 Johnk's generator)
		c = 1. / a;
	    d = 1. / (1 - a);
		for (m = 1; m <= M; m++)
		{
			for (n = 1; n <= N; n++)
			{
				accept = 0;
				do
				{
					u = ranf();
					v = ranf();
			
					x = pow(u, c);
					y = pow(v, d);
			
					if ((x + y) <= 1)
					{
						e = -log(ranf()); 
						r = e * x / (x + y);
						accept = 1;
					}
				}
				while (!accept);
				X_(m,n) = r * b;
			}
		}
	}
	else
	{
		// Use a rejection method for a > 1.
		//% (Devroye, page 410 Best's algorithm)
	    bb = a - 1;
		c = 3 * a - 3./4.;
	
		for (m = 1; m <= M; m++)
		{
			for (n = 1; n <= N; n++)
			{
				accept = 0; 
				counter = 0;
				do
				{
					u = ranf();
					v = ranf();
			
					if (u != 0 && u != 1)
					{
						w = u * (1 - u);
				
						y = sqrt(c / w) * (u - 0.5);
						x = bb + y;
				
						if (x >= 0)
						{
							z = 64 * (w * w * w) * (v * v);
				
							accept = (z <= (1 - 2 * (y * y) / x));
							if (!accept)
							{
								accept = (log(z) <= (2*(bb*log(x/bb)-y)));
							}
						}
					}
					counter++;
				} while (!accept && counter < GAMRND_MAX_REJECT);
				if (counter  >= GAMRND_MAX_REJECT)
				{
					char err[1024];
					sprintf_s (err, 1024, "gamrnd_c: accept = %d, counter = %d, a= %g, b = %g", accept, counter, a, b);
					mexWarnMsgTxt (err);
				}
				X_(m,n) = b * x;
			}
		}
	}
}

// Generate a sample from a normal-inverse Gamma prior
void NIGSample(numtype *Mu, numtype *Ssq, numtype PriorMu, numtype PriorK0, numtype PriorA1, numtype PriorA2, int M, int N)
{
    SafeArray<numtype> Mu_ (Mu,M,N);
    SafeArray<numtype> Ssq_(Ssq,M,N);
    
	int m, n;
	numtype tmp;
	
	for (m = 1; m <= M; m++)
	{
		for (n = 1; n <= N; n++)
		{
			gamrnd_c(&tmp, PriorA1, PriorA2, 1, 1);
			Ssq_(m,n) = 1/tmp;
			Mu_(m,n)  = snorm()*sqrt(Ssq_(m,n)/PriorK0) + PriorMu;
		}
	}
}

/* function X=dirichlet_sample(a, N) */
//function: dirichlet_sample
void dirichlet_sample(numtype *X, const numtype *a, int N, int D)
{
	/*
% DIRICHLET_SAMPLE   Sample from Dirichlet distribution.
%
% DIRICHLET_SAMPLE(a) returns a probability vector sampled from a 
% Dirichlet distribution with parameter vector A.
% DIRICHLET_SAMPLE(a,n) returns N samples, collected into a matrix, each 
% vector having the same orientation as A.
%
%   References:
%      [1]  L. Devroye, "Non-Uniform Random Variate Generation", 
%      Springer-Verlag, 1986

% This is essentially a generalization of the method for Beta rv's.
% Theorem 4.1, p.594
*/
	SafeArray<const numtype> a_(a,  D);
	SafeArray<numtype> X_(X, N, D);

	numtype *Y = new numtype[N*D];
	SafeArray<numtype> Y_(Y, N, D);

	int i, j;
	numtype  sum, tmp;
	for (i = 1; i <= N; i++)
	{
		sum = 0;
		for (j = 1; j <= D; j++)
		{
			gamrnd_c(&tmp, a_( j), 1, 1, 1);
			Y_(i,j) = tmp;
			sum += tmp;
		}
		if (sum == 0)
			sum = 1;
		for (j = 1; j <= D; j++)
		{
			X_(i,j) = Y_(i,j) / sum;
		}
	}
	delete[] Y;
}

// lgamma function
void lgamma_c(numtype *outp, const numtype xx)
{
	double x,y,tmp,ser;
	static double cof[6]={76.18009172947146,-86.50532032941677,
		24.01409824083091,-1.231739572450155,
		0.1208650973866179e-2,-0.5395239384953e-5};
	int j;

	y=x=xx;
	tmp=x+5.5;
	tmp -= (x+0.5)*log(tmp);
	ser=1.000000000190015;
	for (j=0;j<=5;j++) ser += cof[j]/++y;

	*outp = -tmp+log(2.5066282746310005*ser/x);
}

/*
void dirichlet_sample(a,n)
{
//% DIRICHLET_SAMPLE   Sample from Dirichlet distribution.
//%
//% DIRICHLET_SAMPLE(a) returns a probability vector sampled from a 
//% Dirichlet distribution with parameter vector A.
//% DIRICHLET_SAMPLE(a,n) returns N samples, collected into a matrix, each 
//% vector having the same orientation as A.
//%
//%   References:
//%      [1]  L. Devroye, "Non-Uniform Random Variate Generation", 
//%      Springer-Verlag, 1986

//% This is essentially a generalization of the method for Beta rv's.
//% Theorem 4.1, p.594

	int i;
	numtype y, r;
	for (i = 1; i <= n; i++)
	{
		r = 0;
		for (j = 1; j <= D; j++)
		{
			gamrnd_c(&r, a_(j),1, 1, 1);

			r += y;
		}
		r(find(r == 0)) = 1;
		r = y./repmat(r, size(y,2), 1);
	}
}
*/
void SampleICDF(numtype *V, const numtype *X, const numtype *CDF, const int NSamples, const int XLen)
{

	SafeArray<numtype> V_(V,NSamples);

	SafeArray<const numtype> X_(X,XLen);
	SafeArray<const numtype> CDF_(CDF,XLen);

	int i, j;
	numtype r;

	for (i = 1; i <= NSamples; i++)
	{
		V_(i) = Inf;
		r =	ranf();
	
		j = 1; 
		while (V_(i) == Inf && j <= XLen)
		{
			if (CDF_(j) > r)
				V_(i) = X_(j);
			else 
				j++;
		}
		if (j >= XLen)
			V_(i) = X_(XLen);
	}
}

numtype MFGetMu0Norm(const numtype *Mu0, const int T)
{
    SafeArray<const numtype> Mu0_		(Mu0, T);

	numtype sumX = 0, sumX2 = 0;
	for (int j = 1; j <= T; j++)
	{
		sumX += Mu0_(j);
		sumX2 += Mu0_(j)*Mu0_(j);
	}

	return -sumX*sumX/(T*T) + sumX2/T;
}

numtype MFGetMu0NormMAD(const numtype *Mu0, const int T)
{
    SafeArray<const numtype> Mu0_		(Mu0, T);

	numtype med = Med.EvalFast(Mu0, T);
	for (int j = 1; j <= T; j++)
	{
		Med.Set(fabs(Mu0[j-1] - med), j);
	}

	return Med.EvalFast(T);
}
