#ifndef UTILS_H_
#define UTILS_H_

#define  DAC_PI (3.1415926)
#define NUMTYPE_SHIFT (0.4)

#ifdef __cplusplus
extern "C" {
#endif


void RandlibSetSeed(int seed1, int seed2);
//%input seed1
//%input seed2

numtype add_in_log (numtype x, numtype y);
numtype safe_exp (numtype x);

/* function y=GetLogP1Dim(X, Mean, Det, InvCov) */
//function: GetLogP1Dim
void GetLogP1Dim(numtype *y, const numtype *X, numtype Mean, numtype Det, numtype InvCov, const int m, const int n);
//%input X(m,n)
//%input Mean
//%input Det
//%input InvCov
//%output y(m,n)


numtype GetLogP1Dim1(const numtype X, const numtype Mean, const numtype InvCov, const numtype LogInvCov);
//%input X
//%input Mean
//%input InvCov
//%input LogInvCov


/* function y=GetLogPDiag(X, Mean, CovDiag) */
//function: GetLogPDiag
void GetLogPDiag(numtype *y, const numtype *X, const numtype *Mean, const numtype *CovDiag, const int D, const int N);
//%input X(D,N)
//%input Mean(D,1)
//%input CovDiag(D,1)
//%output y(1,N)

/* function y=GetLogPDiagInv(X, Mean, CovDiag) */
//function: GetLogPDiagInv
void GetLogPDiagInv(numtype *y, const numtype *X, const numtype *Mean, const numtype *InvCovDiag, const int D, const int N);
//%input X(D,N)
//%input Mean(D,1)
//%input InvCovDiag(D,1)
//%output y(1,N)

/* function y=GetDLogP1Dim(X, Mean, Det, InvCov) */
//function: GetDLogP1Dim
void GetDLogP1Dim(numtype *y, const numtype *X, numtype Mean, numtype Det, numtype InvCov, const int m, const int n);
//%input X(m,n)
//%input Mean
//%input Det
//%input InvCov
//%output y(m,n)

/* function y=GetP1Dim(X, Mean, Det, InvCov) */
//function: GetP1Dim
void GetP1Dim(numtype *y, const numtype *X, numtype Mean, numtype Det, numtype InvCov, const int m, const int n);
//%input X(m,n)
//%input Mean
//%input Det
//%input InvCov
//%output y(m,n)

/* function y=sigmoid(x) */
//function: sigmoid
void sigmoid(numtype *y, const numtype *x, const int m, const int n);
//%input x(m,n)
//%output y(m,n)

/* function y=dsigmoid(x) */
//function: dsigmoid
void dsigmoid(numtype *y, const numtype *x, const int m, const int n);
//%input x(m,n)
//%output y(m,n)

// return a linear index given three-way index and dimensions
int Sub2Ind (int k, int j, int q, int K, int DExt, int Q);

/* function X=gamrnd_c(a, b, M, N) */
//function: gamrnd
void gamrnd_c(numtype *X, numtype a, numtype b, int M, int N);
//%output X(M,N)

/* function X=dirichlet_sample(a, N) */
//function: dirichlet_sample
void dirichlet_sample(numtype *X, const numtype *a, int N, int D);
//%output X(N,D)

// Sample N values from  a discrete distribution given by p
void sample (numtype *v, const numtype *p, const int N, const int D);
//%output v(N)
//%input  p(D)

// Sample MxN matrix of (mu, ssq) variables from a NIG prior
void NIGSample(numtype *Mu, numtype *Ssq, numtype PriorMu, numtype PriorK0, numtype PriorA1, numtype PriorA2, int M, int N);
//output Mu(M,N)
//output Ssq(M,N)

// lgamma function
void lgamma_c(numtype *y, numtype xx);
//%output y(1)

// sample a random variate using inverse CDF method
void SampleICDF(numtype *V, const numtype *X, const numtype *CDF, const int NSamples, const int XLen);
//%output V(1,NSamples)
//%input  X(1,XLen)
//%input  CDF(1,XLen)

void MeanLogScale (numtype *y, const numtype *x, const int iFirst, const int iLast, const int N);
//%output y(1)
//%input  x(N)

void AddLogScale  (numtype *y, const numtype *x, const int iFirst, const int iLast, const int N);
//%output y(1)
//%input  x(N)

void InvertProb (numtype *LogPX, numtype *LogPZGivenX, const numtype *LogPXGivenZ, const numtype *PZ, const int D);
//%output LogPX(1)
//%output LogPZGivenX(D)
//%input  LogPXGivenZ(D)
//%input  PZ(D)

void InvertProb2 (numtype *LogPX, numtype *LogPZGivenX, const numtype *LogPXGivenZ, const numtype *PZ, const int D, const int N) ;
//%output LogPX(N)
//%output LogPZGivenX(N,D)
//%input  LogPXGivenZ(N,D)
//%input  PZ(D)


void GetLogPIGamma (numtype *p, const numtype *x, const numtype PriorA, const numtype PriorB, const int M, const int N);
//%output p(M,N)
//%input x(M,N)

numtype GetLogPIGamma1 (const numtype x, const numtype PriorA, const numtype PriorB);

void IGammaStat (numtype *m, numtype *v, const numtype PriorA, const numtype PriorB);

void GetLogPGamma (numtype *p, const numtype *x, const numtype PriorA, const numtype PriorB, const int M, const int N);
//%output p(M,N)
//%input x(M,N)

numtype GetLogPGamma1 (const numtype x, const numtype PriorA, const numtype PriorB);

/* function y=GetLogPT(X, nu) */
//function: GetLogPT
void GetLogPT(numtype *Y, const numtype *X, const numtype nu, const numtype mu, const numtype ssq, const int m, const int n);
//%output Y(m,n)
//%input X(m,n)
//%input nu
//%input mu
//%input ssq


numtype GetLogPT1(const numtype X, const numtype nu, const numtype mu, const numtype ssq);
//%input X
//%input nu
//%input mu
//%input ssq

numtype MFGetMu0Norm(const numtype *Mu0, const int T);
//%input Mu0(T)

numtype MFGetMu0NormMAD(const numtype *Mu0, const int T);
//%input Mu0(T)

#ifdef __cplusplus
}
#endif

#endif
