function HPriors = SPMLEDefineHPriors (Data, Config, BConjugateModel, BEstimateMu0, pComp, pCutOff, pAnova, idxClass)

HPriors.PComp = pComp;

[DataMatr, L, R, Len, N, D, Q, X, NRepl, NMaxRepl] = GetDataLRLen(Data);
[M, V] = GetReplVar2(Data);

T = size(Config.CycleDep,1);

idxJAll = []; labelAll = [];
for j = 1 : D
    idxJAll = [idxJAll, [1:NRepl(1,j)] + (j-1)*NMaxRepl];
    labelAll = [labelAll, j*ones(1,NRepl(1,j))];
end

% define candidate periodic genes using raw replicates
if ~exist('idxClass', 'var')
    if ~exist('pAnova', 'var') || ~isfield('pAnova', 'PR')
        pPR = GetANOVAPeriodic(Data, Config);
    else
        pPR = pAnova.PR;
    end
    idxPR = find(pPR < pCutOff.PR);
else
    idxPR = idxClass.PR;
end

% define candidate DE genes
if ~exist('idxClass', 'var')
    if ~exist('pAnova', 'var') || ~isfield('pAnova', 'DE')
        pDE = GetANOVADE(Data);
    else
        pDE = pAnova.DE;
    end
    idxDE  = setdiff(find(pDE < pCutOff.PR), idxPR);
else
    idxDE = idxClass.DE;
end

% define candidate BG genes
if ~exist('idxClass', 'var')
    idxBG = setdiff(find(pDE > pCutOff.BG), union(idxDE, idxPR));
else
    idxBG = idxClass.BG;
end

% build HPriors.BG

% find MLE of igamma parameters
[stat, nElem] = FitIGammaGetStatisticBG2(GetDataSubset(Data, idxBG));
[HPriors.BG.A1(1,1:D), HPriors.BG.A2(1,1:D) ] = FitIGammaGetParams(stat, nElem, 1, [1:length(stat)]);

HPriors.BG.Mu0(1,1:D) = 0;
if BConjugateModel
    % find MLE of the scale parameter K0
    y = DataMatr(idxBG,idxJAll)';
    idxUse = randperm(size(y,2));
    HPriors.BG.K0(1,1:D) = FitNIGScale_GetK(y(:,idxUse(1:min(size(y,2), 2000))), HPriors.BG.Mu0(1), HPriors.BG.A1(1), HPriors.BG.A2(1), [1,100]);
else
    HPriors.BG.SSQ(1,1:D) = 1e-04;
end


% DE model
if length(idxDE) > 0
    [statDE, nElemDE] = FitIGammaGetStatistic(Data);
    for j = 1 : D
        %    [aGlobal(j), bGlobal(j)] = FitIGammaGetParams(statDE, nElemDE, j, [1:N]);
        %    [HPriors.DE.A1(1,j), HPriors.DE.A2(1,j)] = deal(aGlobal(j), bGlobal(j));
        [HPriors.DE.A1(1,j), HPriors.DE.A2(1,j)] = FitIGammaGetParams(statDE, nElemDE, j, idxDE);

        y = DataMatr(idxDE,[1:NRepl(1,j)] + (j-1)*NMaxRepl)';
        if BEstimateMu0
            HPriors.DE.Mu0(1,j) = mean(mean(y));
        else
            HPriors.DE.Mu0(1,j) = 0;
        end
        if BConjugateModel
            HPriors.DE.K0(1,j) = FitNIGScale_GetK(y, HPriors.DE.Mu0(1,j), HPriors.DE.A1(1,j), HPriors.DE.A2(1,j));
        else
            HPriors.DE.SSQ(1,j) = var(mean(y));
        end
    end
else
    warning('SPMLEDefineHPriors: using default priors for DE component, no observations match the criateria for DE');
    HPriors.DE.Mu0(1,1:D) = 0;
    HPriors.DE.K0(1,1:D) = 1;
    HPriors.DE.A1(1,1:D)= 2.5;
    HPriors.DE.A2(1,1:D) = 10;
end

% PR model
if length(idxPR) > 0
    [statPR, nElemPR] = FitIGammaGetStatisticPR(Data, Config);
    for j = 1 : T
        [HPriors.PR.A1(1,j), HPriors.PR.A2(1,j)] = FitIGammaGetParams(statPR, nElemPR, j, idxPR);
        %    [HPriors.PR.A1(1,j), HPriors.PR.A2(1,j)] = deal(aGlobal(j), bGlobal(j));

        idxJ = [];
        for s = 1 : Config.NCycleDep(j)
            t = Config.CycleDep(j,s);
            idxJ = [idxJ, [1:NRepl(1,t)] + (t-1)*NMaxRepl];
        end
        y = DataMatr(idxPR,idxJ)';
        if BEstimateMu0
            HPriors.PR.Mu0(1,j) = mean(mean(y));
        else
            HPriors.PR.Mu0(1,j) = 0;
        end
        if BConjugateModel
            HPriors.PR.K0(1,j) = FitNIGScale_GetK(y, HPriors.PR.Mu0(1,j), HPriors.PR.A1(1,j), HPriors.PR.A2(1,j));
        else
            HPriors.PR.SSQ(1,j) = var(mean(y));
        end
    end
else
    warning('SPMLEDefineHPriors: using default priors for PR component, no observations match the criateria for PR');
    HPriors.PR.Mu0(1,1:T) = 0;
    HPriors.PR.K0(1,1:T) = 1;
    HPriors.PR.A1(1,1:T) = 2.5;
    HPriors.PR.A2(1,1:T) = 10;
end