% add directory to the source search path
addpath('FitIGamma');
addpath('FitNIGScale');
addpath('NIGPeriodic');
addpath('utils');

% setup Data to run periodicity detection
fData = 'LiverData/data.mat';
fprobes = 'LiverData/ProbeID';

%load Data
try
    load(fData, 'Data', 'Config', 'TSegments');
catch
    error(sprintf('Data file %s not found', fData));
end

% load probe ids
try
    load(fprobes, 'ProbeID');
catch
    error(sprintf('ProbeID file %s not found', fprobes));
end

pvaluePR = 0.001;
pvalueDE = 0.01;

% Get mean / variance per time point
[M, V] = GetReplVar2(Data);

% Run empirical Bayes + estimate component posteriors
[HPriors, pKGivenY, LogPY] = estimatePeriodicity(Data, Config, pvaluePR, pvalueDE);

% Order genes according to the posterior probability of periodicity
[dummy, ordSP] = sort(-pKGivenY(:,1));

% visualize the top ranking gene
idxToPlot = ordSP(1);

xLabelInfo.Str = 'Time point'; 
xLabelInfo.Ticks = Data.x; 
xLabelInfo.TickLabels = Data.x

mouse_plot_data (GetDataSubset(Data, idxToPlot), TSegments, false, false, false, true, false, xLabelInfo, false, {ProbeID(idxToPlot)})

% run regular periodogram

% Run periodogram
[I, gTest, pGTest, W] = mf_periodogram (M,[0:size(M,2)-1]);

% order according to the p-value associated with test statistic
[dummy, ordPer] = sort(pGTest);

idxToPlot = ordPer(1);
mouse_plot_data (GetDataSubset(Data, idxToPlot), TSegments, false, false, false, true, false, xLabelInfo, false, {ProbeID(idxToPlot)})

