function [y,v] = mouse_plot_data (data, TSegments, bMeanNormalize, BExp, BPlotReplVar, BPlotRepl, bInteractive, xLabelInfo, yLabelInfo, TitleStr, NFontSize, nLineWidth, nMarkerSize, clust, hFig)

col = 'kgbrmkrgymkrgymkrgymk';

[DataMatr, L, R, Len, N, D, Q, X, NRepl, NMaxRepl] = GetDataLRLen(data);


if exist('hFig')
    figure(hFig);
    hAx = get(hFig, 'CurrentAxes');
    if isempty(hAx)
        hAx = axes;
    end
else
    hFig = figure;
    hAx = axes;
end

if ~exist('NFontSize')
    NFontSize = 14;
end
set(hAx, 'FontSize', NFontSize);

if ~exist('nLineWidth', 'var')
    nLineWidth = 2;
end

if ~exist('nMarkerSize', 'var')
    nMarkerSize = 5;
end

if ~bInteractive
    hold on;
end

if ~exist('clust') | clust == 0
    clust = ones(N, 1);
end

zeros(N, max(Len));
v = zeros(N, max(Len));

for i = 1 : N
    if bInteractive & i > 1
        pause;
        clf;
    end
    hold on;
    if bMeanNormalize
        for jj = 1 : length(TSegments)
            m(i,jj) = 0;
            for j = 1 : length(TSegments{jj})
                m(i,jj) = m(i,jj) + sum(DataMatr(i,[1:NRepl(i,TSegments{jj}(j))] + (TSegments{jj}(j)-1)*NMaxRepl));
            end
            m(i,jj) = m(i,jj) / sum(NRepl(i,TSegments{jj}));
            for j = 1 : length(TSegments{jj})
                DataMatr(i,[1:NRepl(i,TSegments{jj}(j))]+(TSegments{jj}(j)-1)*NMaxRepl) = ...
                    DataMatr(i,[1:NRepl(i,TSegments{jj}(j))]+(TSegments{jj}(j)-1)*NMaxRepl) - m(i,jj);
            end
        end
    end
    
    for j = 1 : Len(i)
        idx = find(~isinf(DataMatr(i,[1:NRepl(i,j)] + (j-1)*NMaxRepl)));
        y(i,j) = mean(DataMatr(i,idx + (j-1)*NMaxRepl));
        v(i,j) = var(DataMatr(i,idx + (j-1)*NMaxRepl));

        % plotting individual replicates
        if (BPlotRepl)
            for r = 1 : NRepl(i,j)
                plot(X(j), DataMatr(i,r + (j-1)*NMaxRepl), strcat(col(r), 'o'), 'LineWidth', nLineWidth, 'MarkerSize', nMarkerSize);
            end
        end
    end
    for jj = 1 : length(TSegments)
        if ~BExp
            plot(X(TSegments{jj}), y(i,TSegments{jj}), strcat(col(clust(i)), '-'), 'LineWidth', nLineWidth);
        else
            plot(X(TSegments{jj}), exp(y(i,TSegments{jj})), strcat(col(clust(i)), '-'), 'LineWidth', nLineWidth);
        end
    end

    % plotting the replicate variance
    if (BPlotReplVar)
        plot(X, sqrt(v(i,1:Len(i))), 'kx');
    end

    % labels / title
    if exist('xLabelInfo')
        if isfield (xLabelInfo, 'Str')
            xlabel(xLabelInfo.Str);
        end
        if isfield (xLabelInfo, 'Ticks')
            set(gca, 'XTick', xLabelInfo.Ticks);
        end
        if isfield (xLabelInfo, 'TickLabels')
            set(gca, 'XTickLabel', xLabelInfo.TickLabels);
        end
    else
        xlabel('Time point');
    end
    
    if exist('yLabelInfo')
        if isfield (yLabelInfo, 'Str')
            ylabel(yLabelInfo.Str);
        end
    else
        ylabel('Observed Intensity');
    end
    
    if exist('TitleStr') & length(TitleStr) == N
        if (length(TitleStr{i}) == 1) || isempty(TitleStr{i}{2})
            title(TitleStr{i}{1});
        else
            title(TitleStr{i});
        end
    elseif exist('TitleStr') & length(TitleStr) == 1
        title(TitleStr);
    end

    xlim([min(data.x), max(data.x)]);
    ylim([min(min(DataMatr(i,~isinf(DataMatr(i,:))&~isnan(DataMatr(i,:))))), max(max(DataMatr(i,~isinf(DataMatr(i,:))&~isnan(DataMatr(i,:)))))]);

%    ManualLabel(i) = input(sprintf('Input class (%d): hc = 1, nhc = 0, do not know = -1:', i))
end
%ManualLabel

return;

